/* JOGAPP: jog dial utility for VAIO

   developed on C1VJ(crusoe)
   Takaya Kinjo <t-kinjo@tc4.so-net.ne.jp> 2000/11/14

   Copyright (C) Takaya Kinjo 2000
   
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

//#define DEBUG
#define APP_MAX 40     /* maximum number of title */
#define NORM 0         /* id for normal mode */
#define VOL 1          /* id for volume mode */
#define LCD 2          /* id for lcd contrast mode */
#define APP_VOL 100    /* not yet inplemented */
#define APP_LCD 100    /* not yet inplemented */
#define JOG_PUSHED 64  /* data when jog pushed */
#define MAX_VOL 50     /* value for maximum volume */
#define MAX_LCD 16     /* value for maximum lcd contrast */
#define PICT_WIDTH 220 /* window width in graphic mode */
#define PICT_HEIGHT 50 /* window height in graphic mode */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

char app_title[APP_MAX][256], app_com[APP_MAX][256];
int app_num      = 0;
int app_num_buff = 0;
int app_max;
int mode         = NORM;
int vol          = 0;
int text_mode    = 0;
int delta        = 0;
GtkWidget *window;
GtkWidget *app_title_c;
GtkWidget *app_title_u;
GtkWidget *app_title_d;
GtkWidget *drawing_area;
GdkGC *g_gc = NULL;
int fd;
char *spic_device = "/dev/spic";
char *ver = "1.0";
//char *fontname = "-marumoji-*-*-*-*-*-*-*-*-*-*-*-*-*";
char *fontname = "-misc-fixed-*-*-*-*-14-70-*-*-*-*-*-*";
// char *fontname = "-adobe-*-*-*-*-*-10-*-*-*-*-*-*-*";

/* truncate app number */
int ring_num( int num ){
  if ( num > app_max ) return( num - app_max -1 );
  if ( num < 0 ) return( app_max + num + 1 );
  return( num );
}
/* limit number for specific app */
int lim_num( int num, int min, int max ){
  if ( num > max ) return( max );
  if ( num < min ) return( min );
  return( num );
}

/* set color value (gc) */
void set_color(gushort r, gushort g, gushort b)
{
  GdkColor  color;

  color.red = r, color.green = g, color.blue = b;
  gdk_color_alloc(gdk_colormap_get_system(),&color);
  gdk_gc_set_foreground(g_gc, &color);
}
/* screen update, used in expose event too */
void draw_picture(GdkDrawable *gd)
{
  GdkPixmap *pixmap;
  GdkPixmap *pixmap2;
  GdkFont *font;
  char str[100];

  gdk_draw_rectangle( gd, drawing_area->style->black_gc,
		      TRUE, 0, 0, PICT_WIDTH, PICT_HEIGHT);

  font = gdk_font_load( fontname );
  set_color( 0x0000, 0x0000, 0xffff); 

  switch( mode ){
  case NORM: /* normal mode */
    set_color( 0x0000, 0x0000, 0xffff); 
    switch( delta ){
    case 2: /* rotate down 1st step */
      gdk_draw_string(gd, font, g_gc, -5, 15-5, 
		      app_title[ring_num(app_num_buff-1)] );
      gdk_draw_string(gd, font, g_gc, 10, 30-5, 
		      app_title[app_num_buff] );
      gdk_draw_string(gd, font, g_gc, 5, 45-5, 
		      app_title[ring_num(app_num_buff+1)] );
      gdk_draw_string(gd, font, g_gc, -10, 60-5, 
		      app_title[ring_num(app_num_buff+2)] );
      pixmap = gdk_pixmap_create_from_xpm(drawing_area->window, NULL, NULL,
					  "./img2.xpm");
      delta = 1;
      break;
    case 1: /* rotate down 2nd step */
      gdk_draw_string(gd, font, g_gc, -10, 15-10,
		      app_title[ring_num(app_num_buff-1)] );
      gdk_draw_string(gd, font, g_gc, 5, 30-10, 
		      app_title[app_num_buff] );
      gdk_draw_string(gd, font, g_gc, 10, 45-10, 
		      app_title[ring_num(app_num_buff+1)] );
      gdk_draw_string(gd, font, g_gc, -5, 60-10, 
		      app_title[ring_num(app_num_buff+2)] );
      pixmap = gdk_pixmap_create_from_xpm(drawing_area->window, NULL, NULL,
					  "./img2.xpm");
      delta = 100; 
      break;
    case 100: /* no rotation but needs to be updated */
      delta = 0; /* continue */
    case 0: /* no rotation, no need to be updated */
      gdk_draw_string(gd, font, g_gc, 0, 15, 
		      app_title[ring_num(app_num-1)] );
      set_color( 0x0000, 0xdfff, 0xdfff); 
      gdk_draw_string(gd, font, g_gc, 15, 30, 
		      app_title[app_num] );
      set_color( 0x0000, 0x0000, 0xffff); 
      gdk_draw_string(gd, font, g_gc, 0, 45,
		      app_title[ring_num(app_num+1)] );
      
      pixmap = gdk_pixmap_create_from_xpm(drawing_area->window, NULL, NULL,
					  "./img1.xpm");
      pixmap2 = gdk_pixmap_create_from_xpm(drawing_area->window, NULL, NULL,
					   "./img4.xpm");
      if (pixmap2) {
	gint w, h;
	gdk_window_get_size(pixmap2, &w, &h);
	gdk_draw_pixmap(gd, g_gc, pixmap2, 0, 0, PICT_WIDTH-35, 20, w, h);
	gdk_pixmap_unref(pixmap2);
      }
      
      app_num_buff = app_num;
      break;
    case -2: /* rotate up 1st step */
      gdk_draw_string(gd, font, g_gc, -10, 15-10, 
		      app_title[ring_num(app_num-1)] );
      gdk_draw_string(gd, font, g_gc, 5, 30-10, 
		      app_title[app_num] );
      gdk_draw_string(gd, font, g_gc, 10, 45-10, 
		      app_title[ring_num(app_num+1)] );
      gdk_draw_string(gd, font, g_gc, -5, 60-10, 
		      app_title[ring_num(app_num+2)] );
      pixmap = gdk_pixmap_create_from_xpm(drawing_area->window, NULL, NULL,
					  "./img3.xpm");
      delta = -1;
      break;
    case -1: /* rotate up 2nd step */
      gdk_draw_string(gd, font, g_gc, -5, 15-5, 
		      app_title[ring_num(app_num-1)] );
      gdk_draw_string(gd, font, g_gc, 10, 30-5, 
		      app_title[app_num] );
      gdk_draw_string(gd, font, g_gc, 5, 45-5, 
		      app_title[ring_num(app_num+1)] );
      gdk_draw_string(gd, font, g_gc, -10, 60-5, 
		      app_title[ring_num(app_num+2)] );
      pixmap = gdk_pixmap_create_from_xpm(drawing_area->window, NULL, NULL,
					  "./img3.xpm");
      delta = 100;
      break;
    default:
    }
    if (pixmap) {
      gint w, h;
      gdk_window_get_size(pixmap, &w, &h);
      gdk_draw_pixmap(gd, g_gc, pixmap, 0, 0, PICT_WIDTH-25, 0, w, h);
      gdk_pixmap_unref(pixmap);
    }
    break;
  case VOL: /* volume mode */
    delta = 0;
    set_color( 0x0000, 0xdfff, 0x0000); 
    gdk_draw_string(gd, font, g_gc, 0, 15, 
		    "SET VOLUME" );
    set_color( 0x0000, 0x0000, 0xffff); 
    gdk_draw_string(gd, font, g_gc, 15, 30, 
		    "RETURN" );
    //    sprintf( str, "[%02d]", vol );
    //gdk_draw_string(gd, font, g_gc, 0, 45,
    //	    str );
    set_color( 0xc000, 0xc000, 0xc000); 
    gdk_draw_rectangle( gd, g_gc,
			TRUE, 0, 35, PICT_WIDTH-25, 14);
    set_color( 0x0000, 0x0000, 0x0000); 
    gdk_draw_rectangle( gd, g_gc,
			TRUE, 1, 36, PICT_WIDTH-27, 12);
    set_color( 0x0000, 0xffff, 0x0000); 
    gdk_draw_rectangle( gd, g_gc,
			TRUE, 1, 36, vol*(PICT_WIDTH-28)/MAX_VOL, 12);
    pixmap2 = gdk_pixmap_create_from_xpm(drawing_area->window, NULL, NULL,
					 "./img4.xpm");
    if (pixmap2) {
      gint w, h;
      gdk_window_get_size(pixmap2, &w, &h);
      gdk_draw_pixmap(gd, g_gc, pixmap2, 0, 0, PICT_WIDTH-35, 20, w, h);
      gdk_pixmap_unref(pixmap2);
    }
    
    break;
  default:
  }
}
/* title display */
void set_title()
{
  int num;
  char str[256];

  if (text_mode){
    switch (mode){
    case NORM:
      sprintf( str, "> %s <", app_title[app_num] );
      gtk_label_set(GTK_LABEL( app_title_c ), str);
      sprintf( str, "%s", app_title[ring_num( app_num + 1 )] );
      gtk_label_set(GTK_LABEL( app_title_d ), str);
      sprintf( str, "%s", app_title[ring_num( app_num - 1)] );
      gtk_label_set(GTK_LABEL( app_title_u ), str);
      break;
      
    case VOL:
      gtk_label_set(GTK_LABEL( app_title_u ), "volume");
      gtk_label_set(GTK_LABEL( app_title_c ), " RETURN <-");
      sprintf( str, "[%3d]", vol );
      gtk_label_set(GTK_LABEL( app_title_d ), str);
      break;

    case LCD:
      gtk_label_set(GTK_LABEL( app_title_u ), "LCD contrast");
      gtk_label_set(GTK_LABEL( app_title_c ), " RETURN <-");
      sprintf( str, "[%3d]", vol );
      gtk_label_set(GTK_LABEL( app_title_d ), str);
      break;
    }
  } else {
    draw_picture(drawing_area->window);
  }
  printf("."); /* I dont know why I need this.. */
               /* but system screwed up without this .. */
}

/* read rc file and store app names, commands */
void read_rc( char *file )
{
  int i;
  char buff[256];
  FILE *fp;

  if ( (fp = fopen( file, "r")) == NULL){
    printf( "file %s not found.\n", file );
    exit( 1 );
  }

  for ( i = 0; i < APP_MAX; i ++){
    if ( fscanf( fp, "%s", &buff ) == EOF ) {
      app_max = i-1;
      break;
    }
    while ( index( buff, ',' ) == NULL){
      strcat( buff, " " );
      strcat( app_title[i], buff );
      fscanf( fp, "%s", &buff );
    }
    strncat( app_title[i], buff, strlen(buff)-1 );

    fscanf( fp, "%s", &buff );
    while ( index( buff, '&' ) == NULL){
      strcat( buff, " " );
      strcat( app_com[i], buff );
      fscanf( fp, "%s", &buff );
    }
    strcat( app_com[i], buff );
#ifdef DEBUG
    printf( "[%s]:[%s]\n", app_title[i], app_com[i] );
#endif
  }
  fclose( fp );
}
/* gtk delete event */
gint delete_event(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  return FALSE;
}
/* gtk destroy event */
void destroy(GtkWidget *widget, gpointer data)
{
  gtk_main_quit ();
}
/* poll jog dial status */
void check_jog()
{
  int jog = 0;

  /* read device */
  if ( read( fd, (char *) &jog, sizeof( int )) < 0){
    printf("%s read error.\n", spic_device);
    close ( fd );
    exit ( 1 );
  }

  /* proc to do if pressed/turned */
  if ( jog ){
    switch( mode ){
    case NORM: /* normal mode */
      if ( jog == JOG_PUSHED ){
	delta = 100; /* prevent freeze */
	if (app_num == APP_VOL) mode = VOL;
	else if (app_num == APP_LCD) mode = LCD;
	else system ( app_com[app_num] ); /* command execution */
      } else {
	if ( jog < 16 ) {
	  app_num  = ring_num( app_num - jog);
	  delta = -2;
	} else { 
	  app_num  = ring_num( app_num - jog+32);
	  delta = 2;
	}
      }
      break;

    case VOL: /* volume mode */
      delta = 100; /* redraw once */
      if ( jog == JOG_PUSHED ) mode = NORM;
      else {
	if ( jog < 16 ) {
	  vol = lim_num( vol-jog, 0, MAX_VOL );
	} else { 
	  vol = lim_num( vol-jog+32, 0, MAX_VOL );
	}
      }
      break;
      
    case LCD: /* lcd mode */
      if ( jog == JOG_PUSHED ) mode = NORM;
      else {
	if ( jog < 16 ) {
	  vol -= jog;
	  if ( vol < 0 ) vol = 0;
	} else { 
	  vol -= jog-32;
	  if ( vol > MAX_LCD ) vol = MAX_LCD;
	}
      }
      break;
    default:
    }
  }
  /* update screen if needed */
  if ( delta != 0 ) set_title();
}
/* gtk expose event handler */ 
gint expose_event(GtkWidget *widget, GdkEventExpose *event)
{
  draw_picture(widget->window);
  return FALSE;
}
void setup_screen()
{
  char label[100];
  GtkWidget *hbox;
  GtkWidget *vbox1;
  GtkWidget *vbox2;
  GtkWidget *button;
  GtkWidget *app_title_space;

  /* set main window */  
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  //gtk_window_set_policy(GTK_WINDOW(window), FALSE, FALSE, FALSE);
  //gtk_widget_show(window);
  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		      GTK_SIGNAL_FUNC (delete_event), NULL);
  gtk_signal_connect (GTK_OBJECT (window), "destroy",
		      GTK_SIGNAL_FUNC (destroy), NULL);
  gtk_container_border_width (GTK_CONTAINER (window), 1);
  
  /* create boxes */
  hbox = gtk_hbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(window), hbox);
  gtk_widget_show(hbox);
  vbox1 = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), vbox1, FALSE, FALSE, 0);
  gtk_widget_show(vbox1);
  vbox2 = gtk_vbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(hbox), vbox2, FALSE, FALSE, 0);
  gtk_widget_show(vbox2);

  /* show title */
  if (text_mode){
    sprintf( label, "  JogApp v%s  ", ver );
    app_title_u = gtk_label_new( label );
    gtk_box_pack_start(GTK_BOX(vbox1), app_title_u, FALSE, FALSE, 0);
    gtk_widget_show( app_title_u );
    app_title_c = gtk_label_new( "  for PCG-C1   " );
    gtk_box_pack_start(GTK_BOX(vbox1), app_title_c, FALSE, FALSE, 0);
    gtk_widget_show( app_title_c );
    app_title_d = gtk_label_new( "<t-kinjo@tc4.so-net.ne.jp>" );
    gtk_box_pack_start(GTK_BOX(vbox1), app_title_d, FALSE, FALSE, 0);
    gtk_widget_show( app_title_d );
  } else {
    /* create graphic area */
    drawing_area = gtk_drawing_area_new();
    gtk_drawing_area_size( GTK_DRAWING_AREA(drawing_area),
			   PICT_WIDTH, PICT_HEIGHT);
    gtk_signal_connect(GTK_OBJECT(drawing_area), "expose_event",
		       (GtkSignalFunc) expose_event, NULL);
    gtk_widget_set_events(drawing_area, GDK_EXPOSURE_MASK);
    gtk_box_pack_start(GTK_BOX(vbox1), drawing_area, FALSE, FALSE, 0);
    gtk_widget_show(drawing_area);
  }
} 
void disp_help()
{
  printf("* JOG-DIAL apprication starter for PCG-C1 *\n");
  printf("  version %s <t-kinjo@tc4.so-net.ne.jp>\n", ver);
  printf("usage:\n");
  printf("jogapp -h       print this message\n");
  printf("       -f file  specify rc_file(default:~/.jogapprc)\n");
  printf("       -t       text mode\n\n");
  printf("rc_file looks like this:\n");
  printf("Title1 in list, command1 &\n");
  printf("Title2 in list, command2 &\n");
  printf("             :\n");
  printf("    ( up to 40 titles)\n        Enjoy VAIO!\n");
}
/**************************
   the main
 **************************/
int main (int argc, char *argv[])
{
  GtkPixmap *dummy;

  int i;
  char fname[256];

  /* set default filename */
  sprintf( fname, "%s/.jogapprc", getenv("HOME") );

  /* this may be bad manner */
  for ( i = 1; i < argc; i ++){
    if ( !strcmp( argv[i], "-h" )){
      disp_help();
      exit( 1 );
    } else if ( !strcmp( argv[i], "-f" )){
      sprintf( fname, "%s", argv[i+1] );
      i ++;
    } else if ( !strcmp( argv[i], "-t" )){
      text_mode = 1;
    }
  }

  /* open spic device */
  if ( (fd = open( spic_device, O_RDONLY)) < 0){
    printf("Could not open %s. Insert module spicdriver first.\n",
	   spic_device );
    exit( 1 );
  }
  /* read rc file */
  read_rc( fname );

  /* gtk init */
  gtk_set_locale();
  gtk_init( &argc, &argv );
  gtk_rc_parse( "./gtkrc" );

  /* prepare screen text/graphic mode */
  setup_screen();

  /* check jog periodically */
  gtk_timeout_add( 50, (GtkFunction)check_jog, dummy );

  gtk_widget_show(window);

  g_gc = gdk_gc_new(window->window);

  gtk_main ();

  return 0;
}







