/***************************************************************************
                          kjog.cpp  -  description
                             -------------------
    begin                : sam mai 26 23:27:06 UTC 2001
    copyright            : (C) 2001 by Shift (Franck Qulain)
    email                : shift@free.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#define JOG_PUSHED 64		/* data when jog pushed */

#include "kjog.h"

// the n of the current application
int app_num = 0;
// the number application in the configuration file
int app_max;
// the link for the device /dec/spic
int fd;

// CONFIG
// Do you want some sound ?
bool withSound;
// The apps and their command
QStringList listCommand;
QStringList listName;

// The jogdial device
char *spic_device = "/dev/spic";

extern "C"
{
  KPanelApplet *init (QWidget * parent, const QString configFile)
  {
    KGlobal::locale ()->insertCatalogue ("KJog");
    return new Kjog (configFile, KPanelApplet::Normal,
		     KPanelApplet::About | KPanelApplet::Help | KPanelApplet::
		     Preferences, parent, "KJog");
  }
}

Kjog::Kjog (const QString & configFile, Type type, int actions,
	    QWidget * parent, const char *name): KPanelApplet (configFile, type, actions, parent, name)
{
  // Get the current application configuration handle
  ksConfig = config ();

  mainView = new myview (this);
  setFixedWidth (70);
  ksConfig->reparseConfiguration ();

  ksConfig->setGroup ("UI");
  withSound = ksConfig->readBoolEntry ("withSound");

  ksConfig->setGroup ("APPLICATIONS");


  listCommand = ksConfig->readListEntry ("command");
  listName = ksConfig->readListEntry ("name");
  if (listCommand.count () < listName.count ())
  {
	  app_max = listCommand.count ();
  }
  else
  {
	  app_max = listName.count ();
  }

  if ((fd = open (spic_device, O_RDONLY)) < 0)
    {
      printf ("Could not open %s. Insert module spicdriver first.\n",
	      spic_device);
      KMessageBox::error (0,
			  i18n
			  ("<center><b>Error</b></center><br> Could not open the jog device<br>Insert module spicdriver first.\n"),
			  i18n ("Erreur de device"), true);
    }

  mainView->show ();

  mainView->TextLabel1->setText ("KJog started");

  /* check jog periodically */
  startTimer (50);
}

Kjog::~Kjog ()
{
  close (fd);
}

void Kjog::timerEvent (QTimerEvent *)
{
  check_jog ();
}

void Kjog::about ()
{
  // Pretty simple, just create the about data and show the about dialog
  KAboutData aboutData ("KJog", I18N_NOOP ("KLassie"), "0.1",
			I18N_NOOP ("VAIO JogDial Utility Applet"),
			KAboutData::License_GPL_V2, "(c) 2001 Franck Qulain",
			I18N_NOOP
			("KJog is a small Kicker plugin to launch programs with your VAIO jogdial\n"));

  aboutData.addAuthor ("Franck Qulain (Shift)", 0, "shift@free.fr");

  KAboutApplication *aboutDlg =
    new KAboutApplication (&aboutData, this, "About KJog");

  aboutDlg->show ();
}

void Kjog::help ()
{
  KMessageBox::information (0, i18n ("Not implemented yet"));
}

void Kjog::preferences ()
{
  KMessageBox::information (0, i18n ("Not implemsented yet"));
}

/*
void Kjog::storePreferences()
{
       QStringList list1;
       QStringList list2;

        // three different ways of appending values:
        list1.append( "xeyes");
        list1 += "gimp";
        list << "xv" << "xterm" << "xcalc";

        ksConfig->setGroup("APPLICATIONS");
        ksConfig->writeEntry("name" ,list1);
        ksConfig->writeEntry("command",list2);

        ksConfig->sync();
}

*/
int Kjog::widthForHeight (int height) const
{
  return width ();
}

int
Kjog::heightForWidth (int width) const
{
  return height ();
}

void
Kjog::resizeEvent (QResizeEvent * _Event)
{

}

/* truncate app number */
int Kjog::ring_num (int num)
{
  if (num >= app_max)
    return (0);
  if (num < 0)
    return (app_max - 1);
  return (num);
}

/* poll jog dial status */
void Kjog::check_jog ()
{
  int jog = 0;

  /* read device */
  if (read (fd, (char *) &jog, sizeof (int)) < 0)
    {
      printf ("%s read error.\n", spic_device);
      mainView->TextLabel1->
	setText ("<font color=red>device unreacheable</font>");
      if ((fd = open (spic_device, O_RDONLY)) >= 0)
	{
	  QString *name = &listName[app_num];
	  mainView->TextLabel1->setText (*name);
	}
    }

  /* proc to do if pressed/turned */
  if (jog)
    {
      printf ("JOG PUSHED\n");
      if (withSound)
	KAudioPlayer::play ("/usr/share/sounds/generic.wav");

      if (jog == JOG_PUSHED)
	{
	  QString *name = &listName[app_num];
	  QString *command = &listCommand[app_num];

	  KProcess run;
	  QStringList qsl = QStringList::split (" ", *command);
	  for (QStringList::Iterator it = qsl.begin (); it != qsl.end ();
	       ++it)
	    {
	      run << (*it);
	    }
	  bool ret = run.start (KProcess::DontCare);
	  if (ret)
	    {
	      mainView->TextLabel1->setText (*command);
	    }
	}
      else
	{
	  if (jog < 16)
	    {
	      app_num = ring_num (app_num - 1);
	      QString *str = &listName[app_num];
	      mainView->TextLabel1->setText (*str);
	    }
	  else
	    {
	      app_num = ring_num (app_num + 1);
	      QString *str = &listName[app_num];
	      mainView->TextLabel1->setText (*str);
	    }
	}
    }
}
